# -*- coding: utf-8 -*-

"""
requests_panel
~~~~~~~~~~~~~~

:copyright: (c) 2014 by .
:license: See LICENSE for more details.

"""

import pkg_resources

try:
    __version__ = pkg_resources.get_distribution('requests_panel').version
except Exception:
    __version__ = 'unknown'


__title__ = 'requests_panel'
__author__ = 'Guillaume Thomas'
__copyright__ = 'Copyright 2014 Guillaume Thomas'

VERSION = __version__



# -*- coding: utf-8 -*-

from django.conf import settings
from debug_toolbar.panels import Panel
from requests_wrapper import _total_seconds, retrieve_all_queries


class RequestsDebugPanel(Panel):
    """
    Panel that displays queries made by Requests backends.
    """
    name = 'Requests'
    template = 'requests_panel/requests_panel.html'
    has_content = True

    def __init__(self, *args, **kwargs):
        super(RequestsDebugPanel, self).__init__(*args, **kwargs)
        self.queries = []

    def _get_query_count(self):
        return

    def nav_title(self):
        return 'Requests Queries'

    def nav_subtitle(self):
        return "%s queries in %.2fs" % (len(self.queries), sum([q.total_seconds for q in self.queries if hasattr(q, "total_seconds")]))

    def url(self):
        return ''

    def title(self):
        return self.nav_title()

    def get_context(self):
        self.queries = retrieve_all_queries()

        return {
            'queries': self.queries,
            'debug': getattr(settings, 'DEBUG', False),
        }

    def process_response(self, request, response):
        if hasattr(self, 'record_stats'):
            self.record_stats(self.get_context())



# -*- coding: utf-8 -*-

import requests
import functools
import copy
try:
    from requests_futures.sessions import FuturesSession
except ImportError as e:
    print("Impossible to load requests_futures, %(e)s" % locals())
    pass

calls = []
_original_methods = dict([(method, getattr(requests, method)) for method in ["get", "post", "head", "put", "delete", "options"]])


@functools.wraps(_original_methods["get"])
def _get(url, *args, **kwargs):
    calls.append(_original_methods["get"](url, *args, **kwargs))
    return calls[-1]


@functools.wraps(_original_methods["post"])
def _post(url, *args, **kwargs):
    calls.append(_original_methods["post"](url, *args, **kwargs))
    return calls[-1]


@functools.wraps(_original_methods["put"])
def _put(url, *args, **kwargs):
    calls.append(_original_methods["put"](url, *args, **kwargs))
    return calls[-1]


@functools.wraps(_original_methods["head"])
def _head(url, *args, **kwargs):
    calls.append(_original_methods["head"](url, *args, **kwargs))
    return calls[-1]


@functools.wraps(_original_methods["delete"])
def _delete(url, *args, **kwargs):
    calls.append(_original_methods["delete"](url, *args, **kwargs))
    return calls[-1]


@functools.wraps(_original_methods["options"])
def _options(url, *args, **kwargs):
    calls.append(_original_methods["options"](url, *args, **kwargs))
    return calls[-1]

requests.get = _get
requests.post = _post
requests.head = _head
requests.put = _put
requests.delete = _delete
requests.options = _options

future_calls = []
if "FuturesSession" in locals():
    _original_methods_futures = dict([(method, getattr(FuturesSession, method)) for method in ["get", "post", "head", "put", "delete", "options"]])

    @functools.wraps(_original_methods_futures["get"])
    def _get_futures(url, *args, **kwargs):
        future_calls.append(_original_methods_futures["get"](url, *args, **kwargs))
        return future_calls[-1]

    @functools.wraps(_original_methods_futures["post"])
    def _post_futures(url, *args, **kwargs):
        future_calls.append(_original_methods_futures["post"](url, *args, **kwargs))
        return future_calls[-1]

    @functools.wraps(_original_methods_futures["put"])
    def _put_futures(url, *args, **kwargs):
        future_calls.append(_original_methods_futures["put"](url, *args, **kwargs))
        return future_calls[-1]

    @functools.wraps(_original_methods_futures["delete"])
    def _delete_futures(url, *args, **kwargs):
        future_calls.append(_original_methods_futures["delete"](url, *args, **kwargs))
        return future_calls[-1]

    @functools.wraps(_original_methods_futures["head"])
    def _head_futures(url, *args, **kwargs):
        future_calls.append(_original_methods_futures["head"](url, *args, **kwargs))
        return future_calls[-1]

    @functools.wraps(_original_methods_futures["options"])
    def _options_futures(url, *args, **kwargs):
        future_calls.append(_original_methods_futures["options"](url, *args, **kwargs))
        return future_calls[-1]

    FuturesSession.get = _get_futures
    FuturesSession.post = _post_futures
    FuturesSession.put = _put_futures
    FuturesSession.delete = _delete_futures
    FuturesSession.head = _head_futures
    FuturesSession.options = _options_futures


def _total_seconds(td):
    return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 1e6) * 1e-6


def retrieve_all_queries():
    global calls, future_calls
    def _result(future):
        try:
            return copy.deepcopy(future.result())
        except Exception as e:
            return None
    responses = copy.deepcopy(calls) + map(_result, filter(lambda future: not future.running(), future_calls))
    
    for response in responses:
        if isinstance(response, requests.models.Response):
            response.total_seconds = _total_seconds(response.elapsed)
    calls = []
    future_calls = []

    return responses



from distutils.core import setup
from setuptools import find_packages


setup(
    name='django-requests-panel',
    version='0.4.0',
    author='Guillaume Thomas',
    author_email='guillaume.thomas642@gmail.com',
    license='LICENCE.txt',
    description='A Django Debug Toolbar panel for requests',
    url='https://github.com/gtnx/django-requests-panel',
    install_requires=map(lambda line: line.strip("\n"), open('requirements.txt', 'r').readlines()),
    include_package_data=True,
    packages=find_packages(),
)



# -*- coding: utf-8 -*-

import datetime
import os
import unittest
import requests
import requests_cache
from requests_wrapper import retrieve_all_queries, _total_seconds
from requests_futures.sessions import FuturesSession


class PanelsTest(unittest.TestCase):
    # def setUp(self):
    #     requests_cache.install_cache(
    #         cache_name=os.path.join(os.path.dirname(__file__), "test"),
    #         allowable_codes=(200, 404),
    #         allowable_methods=('GET', 'POST', 'PUT', 'DELETE', 'HEAD', 'OPTIONS')
    #     )

    def test_all(self):
        self.assertEquals(_total_seconds(datetime.timedelta(hours=1)), 3600)
        self.assertEquals(_total_seconds(datetime.timedelta(minutes=10)), 600)
        requests.get("http://httpbin.org/get")
        requests.post("http://httpbin.org/post")
        requests.put("http://httpbin.org/put")
        requests.delete("http://httpbin.org/delete")
        requests.head("http://httpbin.org/head")
        requests.options("http://httpbin.org/options")
        
        session = FuturesSession()
        futures = []
        futures.append(session.get("http://httpbin.org/get"))
        futures.append(session.post("http://httpbin.org/post"))
        futures.append(session.put("http://httpbin.org/put"))
        futures.append(session.delete("http://httpbin.org/delete"))
        futures.append(session.head("http://httpbin.org/head"))
        futures.append(session.options("http://httpbin.org/options"))

        [f.result() for f in futures]

        queries = retrieve_all_queries()
        self.assertEqual(len(queries), 12)
        for q in queries:
            self.assertTrue(isinstance(q, requests.models.Response))
